# TPK4186 - Advanced Tools for Performance Engineering
# ComplexNumbers.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Required modules
import sys
import math

# Creator
class Complex:
  def __init__(self, rel, img):
    self.rel = rel
    self.img = img

# Accessors
  def GetRealPart(self):
    return self.rel

  def GetImaginaryPart(self):
    return self.img

# Operations
  def Conjugate(self):
    return Complex(self.rel, -self.img)

  def Add(self, other):
    return Complex(self.rel+other.rel, self.img+other.img)

  def Sub(self, other):
    return Complex(self.rel-other.rel, self.img-other.img)

  def Mul(self, other):
    return Complex(self.rel*other.rel - self.img*other.img, self.rel*other.img + self.img*other.rel)

  def Div(self, other):
    # (a1 + b1.i)/(a2 + b2.i) = (a1 + b1.i)x(a2 - b2.i)/(a2 + b2.i)x(a2 - b2.i)
    #                         = [a1*a2 + b1*b2 + (a2*b1-a1*b2).i]/[a2^2 + b2^2]
    d = other.rel**2 + other.img**2
    return Complex((self.rel*other.rel + self.img*other.img)/d, (self.img*other.rel - self.rel*other.img)/d)

  def Pow(self, n):
    if n==0:
      return Complex(1, 0)
    if n==1:
      return self
    k = n//2
    d = self.Pow(k)
    r = d.Mul(d)
    if n%2==0:
      return r
    return self.Mul(r)

# Printing
  def Print(self):
    print(str(self.GetRealPart()) + " + " + str(self.GetImaginaryPart()) + ".i")

# Calculations
c1 = Complex(-4, 1).Div(Complex(-3, 2))
c1.Print()

d21 = Complex(1, 1).Div(Complex(1, -1))
d22 = Complex(1, 2).Mul(Complex(2, 2))
d23 = d21.Sub(d22)
d24 = Complex(3, -1).Div(Complex(1, 1))
c2 = d23.Add(d24)
c2.Print()

d31 = Complex(0, 2).Mul(Complex(-1, 1))
d32 = Complex(math.sqrt(3), 1).Conjugate()
d33 = d32.Pow(3)
d34 = d31.Add(d33)
d35 = Complex(1, 1)
d36 = d35.Conjugate()
d37 = d35.Mul(d36)
c3 = d34.Add(d37)
c3.Print()



