# TPK4186 - Advanced Tools for Performance Engineering
# Shapes.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# 1. Required Modules
# -------------------

import math

# 2. Shapes
# ---------

class Shape:
  def __init__(self, name):
    self.name = name

  def CalculatePerimeter(self):
    return 0.0

  def CalculateArea(self):
    return 0.0

# 3. Circles
# ----------

class Circle(Shape):
  def __init__(self, name, radius):
    Shape.__init__(self, name)
    self.radius = radius

  def CalculatePerimeter(self):
    return 2*math.pi*self.radius

  def CalculateArea(self):
    return math.pi*self.radius**2

# 4. Rectangles
# -------------

class Rectangle(Shape):
  def __init__(self, name, length, width):
    Shape.__init__(self, name)
    self.length = length
    self.width = width

  def CalculatePerimeter(self):
    return 2*(self.length + self.width)

  def CalculateArea(self):
    return self.length * self.width

# 5. Squares
# -----------

class Square(Rectangle):
  def __init__(self, name, length):
    Rectangle.__init__(self, name, length, length)

# 6. Shape Managers
# -----------------

class ShapeManager:
  def __init__(self):
    self.shapes = dict()

  def LookForShape(self, name):
    return self.shapes.get(name, None)

  def AddCircle(self, name, radius):
    self.RemoveShape(name)
    circle = Circle(name, radius)
    self.shapes[name] = circle
    return circle

  def AddRectangle(self, name, length, width):
    self.RemoveShape(name)
    rectangle = Rectangle(name, length, width)
    self.shapes[name] = rectangle
    return rectangle

  def AddSquare(self, name, length):
    self.RemoveShape(name)
    square = Square(name, length)
    self.shapes[name] = square
    return square

  def RemoveShape(self, name):
    self.shapes.pop(name, None)

  def ExtractPerimeters(self):
    perimeters = dict()
    for name in self.shapes:
      shape = self.shapes[name]
      perimeters[name] = shape.CalculatePerimeter()
    return perimeters

  def ExtractAreas(self):
    areas = dict()
    for name in self.shapes:
      shape = self.shapes[name]
      areas[name] = shape.CalculateArea()
    return areas

  def PrintPerimeters(self):
    perimeters = self.ExtractPerimeters()
    print("Perimeters:")
    for name in perimeters:
      print(name + "\t" + str(perimeters[name]))

  def PrintAreas(self):
    areas = self.ExtractAreas()
    print("Areas:")
    for name in areas:
      print(name + "\t" + str(areas[name]))

# 7. Tests
# --------

manager = ShapeManager()
manager.AddCircle("Joe", 2)
manager.AddCircle("Bob", 2)
manager.AddSquare("Bob", 4)
manager.AddRectangle("Jack", 6, 8)
manager.PrintPerimeters()
manager.PrintAreas()

