# TPK4186 - Advanced Tools for Performance Engineering
# StudentCohort.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of contents
# -----------------
# 1. Required modules
# 2. Students
# 3. Cohorts
# 4. Printer
# 5. Queries

# 1. Required modules
# -------------------
import sys
import math

# 2. Students
# -----------

class Student:
  UNDEFINED = 0
  MALE = 1
  FEMALE = 2

  def __init__(self, identificationNumber):
    self.identificationNumber = identificationNumber
    self.familyName = None
    self.firstName = None
    self.gender = Student.UNDEFINED
    self.dayOfBirth = None
    self.monthOfBirth = None
    self.yearOfBirth = None
    self.emailAddress = None
    self.courses = []
  
  def GetIdentificationNumber(self):
    return self.identificationNumber
  def IsIdentificationNumber(self, identificationNumber):
    return self.identificationNumber==identificationNumber

  def GetFamilyName(self):
    return self.familyName
  def SetFamilyName(self, familyName):
    self.familyName = familyName
  def IsFamilyName(self, familyName):
    return self.familyName==familyName

  def GetFirstName(self):
    return self.firstName
  def SetFirstName(self, firstName):
    self.firstName = firstName
  def IsFirstName(self, firstName):
    return self.firstName==firstName

  def GetGender(self):
    return self.gender
  def SetGender(self, gender):
    self.gender = gender

  def GetDayOfBirth(self):
    return self.dayOfBirth
  def GetMonthOfBirth(self):
    return self.monthOfBirth
  def GetYearOfBirth(self):
    return self.yearOfBirth
  def SetDateOfBirth(self, day, month, year):
    self.dayOfBirth = day
    self.monthOfBirth = month
    self.yearOfBirth = year
  def IsDayOfBirth(self, dayOfBirth):
    return self.dayOfBirth==dayOfBirth
  def IsMonthOfBirth(self, monthOfBirth):
    return self.monthOfBirth==monthOfBirth
  def IsYearOfBirth(self, yearOfBirth):
    return self.yearOfBirth==yearOfBirth

  def GetEmailAddress(self):
    return self.emailAddress
  def SetEmailAddress(self, emailAddress):
    self.emailAddress = emailAddress

  def IsRegistered(self, course):
    return course in self.courses
  def Register(self, course):
    if self.IsRegistered(course):
      return
    self.courses.append(course)
  def CancelRegistration(self, course):
    if not self.IsRegistered(course):
      return
    self.courses.remove(course)
  def GetCourses(self):
    return self.courses

# 3. Cohorts
# ----------

class Cohort:
  def __init__(self):
   self.students = dict()

  def LookForStudent(self, identificationNumber):
    return self.students.get(identificationNumber, None)

  def GetStudents(self):
    return self.students.values()

  def NewStudent(self, identificationNumber):
    student = self.LookForStudent(identificationNumber)
    if student!=None:
      return student
    student = Student(identificationNumber)
    self.students[identificationNumber] = student
    return student

  def RecordStudent(self, identificationNumber, firstName, familyName, gender, dayOfBirth, monthOfBirth, yearOfBirth, emailAddress, courses):
    student = self.NewStudent(identificationNumber)
    student.SetFirstName(firstName)
    student.SetFamilyName(familyName)
    student.SetGender(gender)
    student.SetDateOfBirth(dayOfBirth, monthOfBirth, yearOfBirth)
    student.SetEmailAddress(emailAddress)
    for course in courses:
      student.Register(course)

  def RemoveStudent(self, student):
    if not student in self.students:
      return
    self.students.remove(self)

# 4. Queries
# ----------

class Query:
  def __init__(self):
    pass

  def Select(self, students, test, datum):
    result = []
    for student in students:
      if test(student, datum):
        result.append(student)
    return result

  def Union(self, students1, students2):
    result = students1[:]
    for student in students2:
      if not student in students1:
        result.append(student)
    return result

  def Intersection(self, students1, students2):
    result = []
    for student in students1:
      if student in students2:
        result.append(student)
    return result

  def Difference(self, students1, student2):
    result = []
    for student in students1:
      if not student in student2:
        result.append(student)
    return result

  def PrintStudents(self, students):
    for student in students:
      self.PrintStudent(student)
      print("")
    sys.stdout.flush()

  def PrintStudent(self, student):
    print("Number:       \t" + str(student.GetIdentificationNumber()))
    print("Family name:  \t" + student.GetFamilyName())
    print("First name:   \t" + student.GetFirstName())
    if student.GetGender()==Student.FEMALE:
      print("Gender:       \tFemale")
    elif student.GetGender()==Student.MALE:
      print("Gender:       \tMale")
    else:
      print("Gender:       \tUndefined")
    print("Date of birth:\t" + str(student.GetDayOfBirth()) + "/" + str(student.GetMonthOfBirth()) + "/" + str(student.GetYearOfBirth()))
    print("Email address:\t" + student.GetEmailAddress())
    courseList = ""
    for course in student.GetCourses():
      if courseList!="":
        courseList += ", "
      courseList += course
    print("Courses:      \t" + courseList)
